//---------------------------------------------------------------------------
//  Copyright 1998  Formation Design Systems.				All Rights Reserved
//																									http://www.formsys.com
//	John Curtis, Oct 1998.   
//---------------------------------------------------------------------------

#include "stdafx.h"

#ifndef _CCUSTOMBUTTON_
#include "CCustomButton.h"
#endif

#ifndef _CRADIOHELPERS_
#include "CRadioHelpers.h"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////

IMPLEMENT_DYNAMIC(CCustomButton,CButton)

BEGIN_MESSAGE_MAP(CCustomButton,CButton)
	//{{AFX_MSG_MAP(CCustomButton)
	ON_WM_PAINT()
	ON_WM_ENABLE()

	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_MOUSEMOVE()
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()

	ON_MESSAGE(BM_CLICK,OnBMClick)
	ON_MESSAGE(BM_SETCHECK,OnBMSetCheck)
	ON_MESSAGE(BM_SETSTATE,OnBMSetState)
	ON_MESSAGE(BM_GETCHECK,OnBMGetCheck)
	ON_MESSAGE(BM_GETSTATE,OnBMGetState)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
//Helpers to determine the type of the button

BOOL	CCustomButton::IsAuto()const
{
UINT style=GetButtonStyle();
return	(style==BS_AUTOCHECKBOX)	||
				(style==BS_AUTO3STATE) ||
				(style==BS_AUTORADIOBUTTON);
}

BOOL	CCustomButton::Is3State()const
{
UINT style=GetButtonStyle();
return	(style==BS_3STATE)	||
				(style==BS_AUTO3STATE);
}

BOOL	CCustomButton::IsCheck()const
{
UINT style=GetButtonStyle();
return	(style==BS_AUTOCHECKBOX)	||
				(style==BS_CHECKBOX) ||
				Is3State();
}

BOOL	CCustomButton::IsRadio()const
{
UINT style=GetButtonStyle();
return	(style==BS_AUTORADIOBUTTON)	||
				(style==BS_RADIOBUTTON);
}


/////////////////////////////////////////////////////////////////////////////
//	Setting the individual bits in the stored state of the button

void	CCustomButton::SetStateFlag(int bit,BOOL bSet)
	{
	if(bSet)
		m_State|=bit;
	else
		m_State&=~bit;
	Update();//Refresh
	}

/////////////////////////////////////////////////////////////////////////////
// Sets up the CCustomButtonData flags object which defines the state of the
// button and then calls the overridden Draw() method. 

void	CCustomButton::DoDraw(const CRect& r,CDC* pDC)
{
if(pDC)
	{
	//First fill in the client background
	pDC->FillSolidRect(r,::GetSysColor(COLOR_MENU));

	//Set up the drawing flags
	CCustomButtonData data;
	data.m_bSelected=GetSelectionState();
	data.m_bThreeState=IsThreeState();
	data.m_bHilited=GetHiliteState();
	data.m_bFocused=GetFocusState();
	data.m_bEnabled=IsWindowEnabled();
	GetWindowText(data.m_sText);

	Draw(pDC,r,data);
	}
}


/////////////////////////////////////////////////////////////////////////////
//	Provides flicker free drawing of the button.
//	Calls the DoDraw() above, but also buffers the drawing to an offscreen bitmap.

BOOL	CCustomButton::DoBufferDraw(const CRect& r,CDC* pDC)
{
BOOL bBufferOK=false;

if(pDC)
	{
	const BOOL cCanBitBlt=(pDC->GetDeviceCaps(RASTERCAPS) & RC_BITBLT)!=0;
	CDC bufferDC;
	CBitmap bufferBitmap;

	if(cCanBitBlt && bufferDC.CreateCompatibleDC(pDC))
		{
		if(bufferBitmap.CreateCompatibleBitmap(pDC,r.Size().cx,r.Size().cy))
			{
			CBitmap* pOldBitmap=bufferDC.SelectObject(&bufferBitmap);
			if(pOldBitmap!=NULL)
				{
				CRect bufferRect(CPoint(0,0),r.Size());
				DoDraw(bufferRect,&bufferDC);					//Draw into offscreen buffer
				//Flush the buffer to the screen
				pDC->BitBlt(r.left,r.top,r.Width(),r.Height(),&bufferDC,0,0,SRCCOPY);
				bufferDC.SelectObject(pOldBitmap);		//unselect the bitmap
				bBufferOK=true;
				}
			}
		}
	else
		{
		TRACE("Failed to create a compatible DC");
		}
	}
return bBufferOK;
}

/////////////////////////////////////////////////////////////////////////////
//	Called within the class whenever a redraw is needed.

void	CCustomButton::Update(CDC* controlDC)
{
CDC *pDC=controlDC;

CRect	r;
GetClientRect(&r);//Get client rect

if(pDC==NULL)//If controlDC is NULL call GetDC()
	pDC=CWnd::GetDC();

if(pDC!=NULL)
	{
	if(!DoBufferDraw(r,pDC))
		DoDraw(r,pDC);	//If can't buffer then draw direct
		
	//Release DC if we own it
	if(controlDC==NULL && pDC!=NULL)
		CWnd::ReleaseDC(pDC);
	}
}


/////////////////////////////////////////////////////////////////////////////
//	Calls CButton::OnPaint() if not custom drawing otherwise Update()

void CCustomButton::OnPaint() 
{
if(!IsCustom())
	{
	CButton::OnPaint();
	return;
	}

CPaintDC dc(this); // device context for painting
Update(&dc);
}

/////////////////////////////////////////////////////////////////////////////
//	Calls CButton::OnEnable() if not custom drawing otherwise forces a redraw

void CCustomButton::OnEnable(BOOL bEnable) 
{
if(!IsCustom())
	{
	CButton::OnEnable(bEnable);
	return;
	}
Invalidate(false);
}

/////////////////////////////////////////////////////////////////////////////
// Tracking
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
//	Checks if mouse is inside the button's bounds

BOOL CCustomButton::InButtonRect(CPoint pt)const
	{
	CRect	r;
	GetClientRect(&r);
	return r.PtInRect(pt);
	}

/////////////////////////////////////////////////////////////////////////////
//	Called in OnLButtonDown() to track for a possible hit.

void	CCustomButton::StartTracking()
	{
	if(GetCapture()==NULL)
		{
		CWnd::SetFocus();
		SetCapture();
		m_bTracking=TRUE;
		SetHiliteState(true);
		Update();
		}
	}

/////////////////////////////////////////////////////////////////////////////
//	Called in OnLButtonUp() to register a hit. Otherwise to signify 
//	tracking no longer needed.

void	CCustomButton::EndTracking()
	{
	SetHiliteState(false);
	m_bTracking=FALSE;
	if(GetCapture()==this)
		{
		ReleaseCapture();
		Update();
		}
	}


/////////////////////////////////////////////////////////////////////////////
//	Calls CButton::OnMouseMove() if not custom drawing otherwise continues
//	local tracking of the mouse.

void CCustomButton::OnMouseMove(UINT nFlags, CPoint pt) 
{
if(!IsCustom())
	{
	CButton::OnMouseMove(nFlags,pt);
	return;
	}

if(m_bTracking)
	{
	BOOL bInRect=InButtonRect(pt);
	if(bInRect!=GetHiliteState())
		{
		SetHiliteState(bInRect);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
//	Calls CButton::OnLButtonDown() if not custom drawing otherwise starts
//	local tracking of the mouse.

void CCustomButton::OnLButtonDown(UINT nFlags, CPoint pt) 
{
if(!IsCustom())
	{
	CButton::OnLButtonDown(nFlags,pt);
	return;
	}
StartTracking();
}

/////////////////////////////////////////////////////////////////////////////
//	Calls CButton::OnLButtonUp() if not custom drawing otherwise ends
//	local tracking of the mouse. Registers a hit by calling OnBMClick
//	if the mouse is in the button bounds.

void CCustomButton::OnLButtonUp(UINT nFlags, CPoint pt) 
{
if(!IsCustom())
	{
	CButton::OnLButtonUp(nFlags,pt);
	if(InButtonRect(pt))//Clicked
		TellGroupClicked();		
	return;
	}

if(m_bTracking && InButtonRect(pt))
	OnBMClick(0,0);
EndTracking();
}

/////////////////////////////////////////////////////////////////////////////
//	Calls CButton::OnLButtonDblClk() if not custom drawing otherwise OnLButtonDown()
//	which will start local tracking of the mouse.

void CCustomButton::OnLButtonDblClk(UINT nFlags, CPoint pt) 
{
if(!IsCustom())
	{
	CButton::OnLButtonDblClk(nFlags,pt);
	return;
	}
OnLButtonDown(nFlags,pt);
}

/////////////////////////////////////////////////////////////////////////////
//	Calls CButton::OnSetFocus() if not custom drawing otherwise set
//	the cached focus state.

void CCustomButton::OnSetFocus(CWnd* pWnd)
	{
	if(!IsCustom())
		{
		CButton::OnSetFocus(pWnd);
		return;
		}
	SetFocusState(TRUE);
	Update();
	}

/////////////////////////////////////////////////////////////////////////////
//	Calls CButton::OnKillFocus() if not custom drawing otherwise set
//	the cached focus state.

void CCustomButton::OnKillFocus(CWnd* pWnd) 
	{
	if(!IsCustom())
		{
		CButton::OnKillFocus(pWnd);
		return;
		}
	SetFocusState(FALSE);
	EndTracking();	//eg. If key pressed while tracking
	Update();
	}

/////////////////////////////////////////////////////////////////////////////
//	Send a BN_CLICKED message to any encompassing groupboxes.

void	CCustomButton::TellGroupClicked()
{
if(IsRadio())
	{
	CRadioHelpers::TellGroupClicked(this);
	}
}

/////////////////////////////////////////////////////////////////////////////
//	Calls Default() for the default window behaviour if not custom drawing 
//	otherwise handle the hit.

//An application sends a BM_CLICK message to simulate the user clicking a button. 
//This message causes the button to receive a WM_LBUTTONDOWN and a WM_LBUTTONUP message, 
//and the button's parent window to receive a BN_CLICKED notification message.

LRESULT CCustomButton::OnBMClick(WPARAM,LPARAM)
{
	if(!IsCustom())
		{
		return CButton::Default();
		}

	if(IsCheck())
		{
		if(Is3State() && IsThreeState())
			{
			SetThreeState(FALSE);
			SetSelectionState(FALSE);
			}
		else if(!GetSelectionState())
			{
			SetSelectionState(TRUE);
			}
		else if(GetSelectionState())
			{
			if(Is3State())
				SetThreeState(TRUE);
			else
				SetSelectionState(FALSE);
			}
		}
	else if(IsRadio())
		{
		if(!GetSelectionState())
			{
			if(IsAuto())
				{
				SetSelectionState(1);
				CRadioHelpers::HitRadioItem(this);//Update the radio group
				}
			}
		TellGroupClicked();
		}
	if(!GetFocusState())	//set focus if not already
		CWnd::SetFocus();

	CRadioHelpers::TellParentClicked(this);
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
//	Calls Default() for the default window behaviour if not custom drawing 
//	otherwise sets the cached check state and updates.

LRESULT CCustomButton::OnBMSetCheck(WPARAM checkState,LPARAM)
	{
	if(!IsCustom())
		{
		return CButton::Default();
		}

	EndTracking();

	if(checkState==2)
		SetThreeState(TRUE);
	else
		SetSelectionState(checkState);

	return 0;
	}

/////////////////////////////////////////////////////////////////////////////
//	Calls Default() for the default window behaviour if not custom drawing 
//	otherwise sets the cached state and updates.

LRESULT CCustomButton::OnBMSetState(WPARAM state,LPARAM)
	{
	if(!IsCustom())
		{
		return CButton::Default();
		}

	EndTracking();
	m_State=state;
	return 0;
	}

/////////////////////////////////////////////////////////////////////////////
//	Calls Default() for the default window behaviour if not custom drawing 
//	otherwise gets the cached check state.

LRESULT CCustomButton::OnBMGetCheck(WPARAM,LPARAM)
	{
	if(!IsCustom())
		{
		return CButton::Default();
		}
	return m_State & 0x0003;
	}

/////////////////////////////////////////////////////////////////////////////
//	Calls Default() for the default window behaviour if not custom drawing 
//	otherwise gets the cached state.

LRESULT CCustomButton::OnBMGetState(WPARAM,LPARAM)
	{
	if(!IsCustom())
		{
		return CButton::Default();
		}
	return m_State;
	}