//---------------------------------------------------------------------------
//  Copyright 1998  Formation Design Systems.				All Rights Reserved
//																									http://www.formsys.com
//	John Curtis, Oct 1998.   
//---------------------------------------------------------------------------

#ifndef _CCUSTOMBUTTON_
#define _CCUSTOMBUTTON_


///////////////////////////////////////////////////////////////////////
//	Class CCustomButtonData
//
//  Contains the state data required to draw the button
//	This class is passed to the CCustomButton virtual method Draw()
///////////////////////////////////////////////////////////////////////

class CCustomButtonData
{
public:
	BOOL		m_bSelected;
	BOOL		m_bThreeState;
	BOOL		m_bHilited;
	BOOL		m_bFocused;
	BOOL		m_bEnabled;

	CString m_sText;
	};

///////////////////////////////////////////////////////////////////////
//	Class CCustomButton
//	
//	Contains a class for generic custom drawing of a button
//	To custom draw a button override IsCustom() so that it returns TRUE and
//	then override the Draw() method to do the drawing.
///////////////////////////////////////////////////////////////////////

class CCustomButton :public CButton
{
	DECLARE_DYNAMIC(CCustomButton)
public:
	CCustomButton():m_State(0),m_bTracking(FALSE){}
	virtual ~CCustomButton(){}

protected:
	virtual	void	Draw(CDC*,CRect,const CCustomButtonData&){}
	virtual	BOOL	IsCustom()const{return FALSE;}

	//{{AFX_MSG(CCustomButton)
protected:
	//All these methods/messages need to be overridden/subclassed to
	//avoid any drawing by the underlying window object. 
	afx_msg void OnPaint();
	afx_msg void OnEnable(BOOL bEnable);

	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);

	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnKillFocus(CWnd* pNewWnd);

	LRESULT OnBMClick(WPARAM,LPARAM);
	LRESULT OnBMSetCheck(WPARAM wParam,LPARAM lParam);
	LRESULT OnBMSetState(WPARAM wParam,LPARAM lParam);
	LRESULT OnBMGetCheck(WPARAM,LPARAM);
	LRESULT OnBMGetState(WPARAM,LPARAM);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

protected:
	//Access to button state
	void	SetStateFlag(int bit,BOOL bSet);
	BOOL	GetStateFlag(int bit)const{return (m_State & bit)!=0;}

	void	SetSelectionState(BOOL bSel)		{SetStateFlag(0x0001,bSel);}
	void	SetThreeState(BOOL bThreeState)	{SetStateFlag(0x0002,bThreeState);}
	void	SetHiliteState(BOOL bHilite)		{SetStateFlag(0x0004,bHilite);}
	void	SetFocusState(BOOL bFocus)			{SetStateFlag(0x0008,bFocus);}

	BOOL	GetSelectionState()const{return GetStateFlag(0x0001);}
	BOOL	IsThreeState()const			{return GetStateFlag(0x0002);}
	BOOL	GetHiliteState()const		{return GetStateFlag(0x0004);}
	BOOL	GetFocusState()const		{return	GetStateFlag(0x0008);}

	//Support for Hit tracking
	BOOL	InButtonRect(CPoint pt)const;	//Is the pt in the item rect
	void	StartTracking();							//Begin tracking on a MouseDown.
	void	EndTracking();								//End tracking

	//Send a BN_CLICKED to any Groupbox that contains this button
	void	TellGroupClicked();						

	//Drawing
	void	Update(CDC* pDC=NULL);				
	void	DoDraw(const CRect& rClient,CDC* pDC=NULL);				//Call the overridden virtual method Draw()
	BOOL	DoBufferDraw(const CRect& rClient,CDC* pDC=NULL);	//Buffered call to DoDraw()

	//Style access
	BOOL	IsAuto()const;
	BOOL	IsCheck()const;
	BOOL	IsRadio()const;
	BOOL	Is3State()const;

protected:
	BOOL m_bTracking;		//Hit tracking
	int	 m_State;				//Current button state
};

#endif
