//---------------------------------------------------------------------------
//  Copyright 1998  Formation Design Systems.				All Rights Reserved
//																									http://www.formsys.com
//	John Curtis, Oct 1998.   
//---------------------------------------------------------------------------

#include "stdafx.h"

#ifndef _CCUSTOMICONBUTTON_
#include "CCustomIconButton.h"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC(CCustomIconButton,CCustomButton)

BEGIN_MESSAGE_MAP(CCustomIconButton,CCustomButton)
	//{{AFX_MSG_MAP(CCustomIconButton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////

static CRect GetBorderRect(const CRect& rButton,CSize sInflate)
{	
	CRect r=rButton;
	r.InflateRect(sInflate);
	return r;
}

/////////////////////////////////////////////////////////////////////////////
//Create an 'icon' from a toolbar bitmap

BOOL CCustomIconButton::LoadIcon(UINT nBitmapID,int nOffset,CSize sz)
{
	BOOL bLoadOK=FALSE;
	CBitmap bmToolbar;

	if(bmToolbar.LoadBitmap(nBitmapID))// load toolbar bitmap
		{
		if(m_Image.Create(sz.cx,sz.cy,ILC_MASK,0,10))//Create imagelist from bitmap
			{
			//Add bitmap to the image list. Use default button color as mask color
			m_Image.Add(&bmToolbar,GetSysColor(COLOR_3DFACE));//mask color
			m_Size=sz;
			
			nOffset--; //convert to 0 based

			//nOffset must be a valid offset
			ASSERT(nOffset>=0 && nOffset<m_Image.GetImageCount());

			if(nOffset>=0 && nOffset<m_Image.GetImageCount())
				{
				//Remove unneeded images
				for(int i=m_Image.GetImageCount()-1;i>=0;i--)
					{
					if(i!=nOffset)
						m_Image.Remove(i);	
					}
				}
			//Should be left with the one image
			ASSERT(m_Image.GetImageCount()==1);
			}
		}
	return bLoadOK;
}

/////////////////////////////////////////////////////////////////////////////
//Draw the button based on the current state as defined in flags.

void	CCustomIconButton::Draw(CDC* pDC,CRect rBounds,const CCustomButtonData& flags)
{
const CSize cFocusMargin(5,5);		// number of pixels wider focus is than icon
const CSize cButtonMargin(8,8);		// number of pixels wider button is than icon
const CSize cSelectionMargin(3,3);// number of pixels wider selection is than icon

if(pDC!=NULL && m_Image.GetImageCount()==1)
	{
	CRect r;
	CBrush* dummyBrush=NULL;
	CPoint centre=rBounds.CenterPoint();

	//Centre the icon 
	r.left	=centre.x-m_Size.cx/2;
	r.right	=centre.x+m_Size.cx/2;
	r.top		=centre.y-m_Size.cy/2;
	r.bottom=centre.y+m_Size.cy/2;

	//Paint icon
		
	//Draw button edge depending on if hilited or not, draw pushed-in or popped-out edge
	pDC->DrawEdge(GetBorderRect(r,cButtonMargin),
								flags.m_bHilited ? BDR_SUNKENOUTER:BDR_RAISEDINNER,BF_RECT);
	
	//If enable draw selection and standard icon
	if(flags.m_bEnabled) 
		{
		if(flags.m_bSelected) 
			{
			pDC->FillSolidRect(GetBorderRect(r,cSelectionMargin),GetSysColor(COLOR_BTNHIGHLIGHT));
			pDC->DrawEdge(GetBorderRect(r,cSelectionMargin),BDR_SUNKENOUTER,BF_RECT);
			}
		m_Image.Draw(pDC,0,r.TopLeft(),ILD_TRANSPARENT);//Use ImageList draw routine
		} 
	else //Draw disabled
		{
		HICON hIcon=m_Image.ExtractIcon(0);
		pDC->DrawState(r.TopLeft(),m_Size,hIcon,DSS_DISABLED,dummyBrush);
		DestroyIcon(hIcon);
		}

	//Draw focus rect
	if(flags.m_bFocused)
		{
		pDC->DrawFocusRect(GetBorderRect(r,cFocusMargin));
		}
	} 
}



