//---------------------------------------------------------------------------
//  Copyright 1998  Formation Design Systems.				All Rights Reserved
//																									http://www.formsys.com
//	John Curtis, Oct 1998.   
//---------------------------------------------------------------------------

#include "stdafx.h"

#ifndef _CRADIOGROUP_
#include "CRadioGroup.h"
#endif

#ifndef _CRADIOHELPERS_
#include "CRadioHelpers.h"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


///////////////////////////////////////////////////////////////////////
//	Class CRadioGroup -- control for establishing Radio hits
///////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////
//Used to set the active radio by index.The index is 1 based and 
//the order is in the order they are created in the resource definition.

void	CRadioGroup::SetRadio(int whichRadio)
	{
	CRect rect;
	GetWindowRect(&rect);
	DoRadioGroup(GetParent(),rect,FALSE,whichRadio);
	}

///////////////////////////////////////////////////////////////////////
//Return the 1 based index of the currently active radio.

int	CRadioGroup::GetRadio()const
	{
	int radioActive=0;
	CRect rect;
	GetWindowRect(&rect);
	radioActive=DoRadioGroup(GetParent(),rect,TRUE);

	return radioActive;
	}

///////////////////////////////////////////////////////////////////////
//Deselect all radios bounded by the groupbox

void	CRadioGroup::UnselectAll()	
	{
	CRect rect;
	GetWindowRect(&rect);
	DoRadioGroup(GetParent(),rect,FALSE,0);
	}

///////////////////////////////////////////////////////////////////////
//Helper method to set/get radios bounded by the groupbox.
//rBounds:		bounds of the groupbox
//bGetIndex:	getting or setting radio
//index:			index of the radio button (only used if bGetIndex==FALSE)

long CRadioGroup::DoRadioGroup(const CWnd* dialog,const CRect& rBounds,BOOL bGetIndex,int index)
	{
	CRect rect;
	long radioNo=0;
	BOOL bContinue=TRUE;
	CWnd* wnd =NULL;
	
	if(dialog)
		{
		radioNo=0;
		wnd=dialog->GetWindow(GW_CHILD);
		for(;wnd!=NULL && bContinue && wnd->GetParent()==dialog;wnd=wnd->GetNextWindow())	// walk all children in group
			{
			wnd->GetWindowRect(&rect);
			if(rect.IntersectRect(rect,rBounds))
				{
				if(CRadioHelpers::IsAutoRadio(wnd))
					{
					radioNo++;
					if(bGetIndex)		// thus get active
						{
						if(wnd->SendMessage(BM_GETCHECK,0,0L)!=0)//Have the active button
							bContinue=FALSE;	
						}
					else// set the button
						{
						wnd->SendMessage(BM_SETCHECK,radioNo==index,0L);
						}
					}
				}
			} 
		if(bContinue && bGetIndex)//Didn't find set radio
			{
			radioNo=0;
			TRACE("CRadioGroup::DoRadioGroup: Didn't find set radio in group.");
			}
		}
	return radioNo;
	}

BEGIN_MESSAGE_MAP(CRadioGroup,CButton)
	//{{AFX_MSG_MAP(CRadioGroup)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()




