//---------------------------------------------------------------------------
//  Copyright 1998  Formation Design Systems.				All Rights Reserved
//																									http://www.formsys.com
//	John Curtis, Oct 1998.   
//---------------------------------------------------------------------------

#include "stdafx.h"

#ifndef _CRADIOHELPERS_
#include "CRadioHelpers.h"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


///////////////////////////////////////////////////////////////////////
//	Class CRadioHelpers -- namespace for generic radio button helpers
///////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////
//Is this CWnd a radio button

BOOL CRadioHelpers::IsRadio(CWnd* pRadio)
	{
	BOOL isRadio=FALSE;
	if(pRadio)
		isRadio=pRadio->SendMessage(WM_GETDLGCODE,0,0L) & DLGC_RADIOBUTTON;
	return isRadio;
	}


///////////////////////////////////////////////////////////////////////
//Is this CWnd an auto radio button

BOOL CRadioHelpers::IsAutoRadio(CWnd* pRadio)
	{
	BOOL bAutoRadio=FALSE;
	if(pRadio)
		bAutoRadio=(::GetWindowLong(pRadio->GetSafeHwnd(),GWL_STYLE) & BS_AUTORADIOBUTTON)!=0;
	return bAutoRadio;
	}


///////////////////////////////////////////////////////////////////////
//Is this CWnd a radio button and soes it have the group flag set

BOOL CRadioHelpers::IsRadioGroup(CWnd* pRadio)
	{
	BOOL bIsGroup=FALSE;
	if(pRadio)
		bIsGroup=::GetWindowLong(pRadio->GetSafeHwnd(),GWL_STYLE) & WS_GROUP;
	return IsRadio(pRadio) && bIsGroup;
	}


///////////////////////////////////////////////////////////////////////
//Set the state of an auto-radio button
//Does nothing if not an auto radio

void CRadioHelpers::SetAutoRadio(CWnd* pRadio,BOOL state)
	{
	if(IsAutoRadio(pRadio))
		pRadio->SendMessage(BM_SETCHECK,state,0L);
	}


///////////////////////////////////////////////////////////////////////
//Get the state of an auto-radio button
//Returns 0 if not an auto radio

int CRadioHelpers::GetAutoRadio(CWnd* pRadio)
	{
	int nCheckState=0;
	if(IsAutoRadio(pRadio))
		nCheckState=pRadio->SendMessage(BM_GETCHECK,0,0L);
	return nCheckState;
	}


///////////////////////////////////////////////////////////////////////
//Is this CWnd a groupbox

BOOL CRadioHelpers::IsGroupBox(CWnd* pButton)
	{
	BOOL bGroupBox=FALSE;
	if(pButton!=NULL)
		bGroupBox=(::GetWindowLong(pButton->GetSafeHwnd(),GWL_STYLE) & BS_GROUPBOX)!=0;

	return bGroupBox;
	}


///////////////////////////////////////////////////////////////////////
//Either find an auto radio group or the first non radio control

CWnd* CRadioHelpers::FindStartOfRadioGroup(CWnd* pStartRadio)
	{
	CWnd* pDialog=NULL;
	CWnd* pRadio =NULL;
	CWnd* pNextRadio =NULL;

	if(pStartRadio)
		{
		pDialog=pStartRadio->GetParent();
		if(pDialog)
			{
			for(pRadio=pStartRadio;pRadio!=NULL && !IsRadioGroup(pRadio);)	// walk back till find start of group
				{
				pNextRadio=pDialog->GetNextDlgGroupItem(pRadio,TRUE);//Get previous
				
				if(!IsRadio(pNextRadio))
					{
					break;
					}

				if(pNextRadio==pRadio || pNextRadio==pStartRadio)
					{
					TRACE("FindStartOfRadioGroup : Empty group\n");
					pRadio=NULL;
					}
				else
					{
					pRadio=pNextRadio;
					}
				}
			}
		}
	return pRadio;
	}


///////////////////////////////////////////////////////////////////////
//Respond to a click in a radio button

void CRadioHelpers::HitRadioItem(CWnd* pHitRadio)
	{
	CWnd* pStartRadio =FindStartOfRadioGroup(pHitRadio);
	CWnd* pDialog=NULL;
	CWnd* pRadio =NULL;

	if(pHitRadio && pStartRadio)
		{
		pDialog=pHitRadio->GetParent();
		if(pDialog)
			{
			for(pRadio=pStartRadio;pRadio!=NULL && IsRadio(pRadio);)	// walk back till find start of group
				{
				if(IsRadio(pRadio))
					{
					if(pRadio==pHitRadio && IsAutoRadio(pRadio))
						{
						pRadio->SendMessage(BM_SETCHECK,1,0L);
						}
					else
						{
						pRadio->SendMessage(BM_SETCHECK,0,0L);	//turn off others
						}
					}
				pRadio=pDialog->GetNextDlgGroupItem(pRadio);
				if(pRadio==pStartRadio || !IsRadio(pRadio))
					break;
				}
			}
		}
	}


///////////////////////////////////////////////////////////////////////
//Send a BN_CLICKED message to the button's parent

void CRadioHelpers::TellParentClicked(CWnd* pButton) 
	{
	if(pButton)
		{
		CWnd* pParent=pButton->GetParent();
		int nControlID=pButton->GetDlgCtrlID();
		HWND hWnd=pButton->GetSafeHwnd();
		if(pParent!=NULL && hWnd!=0)
			pParent->SendMessage(WM_COMMAND,MAKEWPARAM(nControlID,BN_CLICKED),(LPARAM)hWnd);
		}
	}


///////////////////////////////////////////////////////////////////////
//Send a BN_CLICKED message to all encompassing groupbox's
	
void CRadioHelpers::TellGroupClicked(CWnd* pButton) 
	{
	if(pButton)
		{
		CRect rButton;
		pButton->GetWindowRect(rButton);

		CWnd* pFirstItem=pButton->GetWindow(GW_HWNDFIRST);//Get first sibling
		CWnd* pLastItem	=pButton->GetWindow(GW_HWNDLAST);	//Get last sibling

		if(pFirstItem && pLastItem)// walk through the items to find groupbox's
			{
			for(CWnd* pItem=pFirstItem;pItem!=NULL;pItem=pItem->GetWindow(GW_HWNDNEXT))	
				{				
				if(IsGroupBox(pItem))
					{
					CRect rGroupBox;
					pItem->GetWindowRect(rGroupBox);
					if(rGroupBox.IntersectRect(rGroupBox,rButton))
						{
						TellParentClicked(pItem);
						}
					}
				if(pItem==pLastItem)
					break;
				}
			}
		}
	}
